/*
 *  setoutdircommand.cpp
 *  Mothur
 *
 *  Created by westcott on 1/21/10.
 *  Copyright 2010 Schloss Lab. All rights reserved.
 *
 */

#include "setoutdircommand.h"

//**********************************************************************************************************************

SetOutputDirectoryCommand::SetOutputDirectoryCommand(string option){
	try {
		abort = false;
		
		//allow user to run help
		if(option == "help") { help(); abort = true; }
		
		else {
			//valid paramters for this command
			string Array[] =  {"output"};
			vector<string> myArray (Array, Array+(sizeof(Array)/sizeof(string)));
			
			OptionParser parser(option);
			map<string, string> parameters = parser.getParameters();
			
			ValidParameters validParameter;
			//check to make sure all parameters are valid for command
			for (map<string,string>::iterator it = parameters.begin(); it != parameters.end(); it++) { 
				if (validParameter.isValidParameter(it->first, myArray, it->second) != true) {  abort = true;  }
			}
		
			output = validParameter.validFile(parameters, "output", false);			
			if (output == "not found") { 
				mothurOut("The output parameter is required for the set.outdir command."); mothurOutEndLine(); abort = true;
			}
		}
	}
	catch(exception& e) {
		errorOut(e, "SetOutputDirectoryCommand", "SetOutputDirectoryCommand");
		exit(1);
	}
}
//**********************************************************************************************************************

void SetOutputDirectoryCommand::help(){
	try {
		mothurOut("The set.outdir command is used to direct the output files generated by mothur to a specific place, the directory must exist.\n");
		mothurOut("The set.outdir command parameter is output and is required.\n");
		mothurOut("To return the output to the same directory as the input files you may enter: output=clear.\n");
		mothurOut("The set.outdir command should be in the following format: set.outdir(output=yourOutputDirectory).\n");
		mothurOut("Example set.outdir(output=/Users/lab/desktop/outputs).\n");
		mothurOut("Note: No spaces between parameter labels (i.e. output), '=' and parameters (i.e.yourOutputDirectory).\n\n");
	}
	catch(exception& e) {
		errorOut(e, "SetOutputDirectoryCommand", "help");
		exit(1);
	}
}
//**********************************************************************************************************************

SetOutputDirectoryCommand::~SetOutputDirectoryCommand(){}

//**********************************************************************************************************************

int SetOutputDirectoryCommand::execute(){
	try {
		
		if (abort == true) { return 0; }
		
		commandFactory = CommandFactory::getInstance();
	
		if (output == "clear") {  output = "";  commandFactory->setOutputDirectory(output);  }
		else {
			//add / to name if needed
			string lastChar = output.substr(output.length()-1);
			if (lastChar != "/") { output += "/"; }
								
			//test to make sure directory exists
			string outTemp = output + "temp";
			ofstream out;
			out.open(outTemp.c_str(), ios::trunc);
			if(!out) {
				mothurOut("directory does not exist or is not writable."); mothurOutEndLine(); return 0;
			}else{
				remove(outTemp.c_str());
				mothurOut("Changing output directory to " + output); mothurOutEndLine();  
				commandFactory->setOutputDirectory(output);
			}
		}
		
		return 0;
	}
	catch(exception& e) {
		errorOut(e, "SetOutputDirectoryCommand", "SetOutputDirectoryCommand");
		exit(1);
	}
}
//**********************************************************************************************************************/
